"use strict";

var _tap = require("tap");

var _ramda = require("ramda");

var _istype = require("@unction/istype");

var _istype2 = _interopRequireDefault(_istype);

var _ispresent = require("@unction/ispresent");

var _ispresent2 = _interopRequireDefault(_ispresent);

var _ispopulated = require("@unction/ispopulated");

var _ispopulated2 = _interopRequireDefault(_ispopulated);

var _endswith = require("@unction/endswith");

var _endswith2 = _interopRequireDefault(_endswith);

var _lackstext = require("@unction/lackstext");

var _lackstext2 = _interopRequireDefault(_lackstext);

var _ = require("./");

var _2 = _interopRequireDefault(_);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/* eslint-disable flowtype/require-parameter-type, flowtype/require-return-type, no-magic-numbers */
var accountAttributes = (0, _2.default)({
  attributes: {
    username: { source: (0, _ramda.pipe)((0, _ramda.prop)("email"), (0, _ramda.split)("@"), _ramda.head) },
    email: { source: (0, _ramda.prop)("email") },
    name: { source: (0, _ramda.prop)("name") },
    age: {
      source: (0, _ramda.prop)("age"),
      coerce: function coerce(value) {
        return parseInt(value, 10);
      }
    },
    createdAt: {
      source: (0, _ramda.prop)("created-at"),
      coerce: function coerce(value) {
        return new Date(value);
      }
    }
  },
  validations: {
    email: {
      presenceRequired: (0, _ramda.propSatisfies)(_ispresent2.default, "email"),
      stringRequired: (0, _ramda.propSatisfies)((0, _istype2.default)("String"), "age"),
      googleEmailRequired: (0, _ramda.propSatisfies)((0, _ramda.both)(_ispopulated2.default, (0, _endswith2.default)("@gmail.com")), "email"),
      noSuffixUsername: (0, _ramda.propSatisfies)((0, _ramda.both)(_ispopulated2.default, (0, _lackstext2.default)("+")), "email")
    },
    name: {
      presenceRequired: (0, _ramda.propSatisfies)(_ispresent2.default, "name"),
      stringRequired: (0, _ramda.propSatisfies)((0, _istype2.default)("String"), "age")
    },
    age: {
      presenceRequired: (0, _ramda.propSatisfies)(_ispresent2.default, "age"),
      numberRequired: (0, _ramda.propSatisfies)((0, _istype2.default)("Number"), "age"),
      olderThanMaximumAge: (0, _ramda.propSatisfies)((0, _ramda.both)(_ispresent2.default, (0, _ramda.lt)(30)), "age")
    }
  }
});
var accountResource = (0, _2.default)({
  attributes: {
    id: { source: (0, _ramda.prop)("id") },
    type: { source: (0, _ramda.prop)("type") },
    attributes: {
      source: (0, _ramda.prop)("attributes"),
      coerce: accountAttributes
    }
  },
  validations: {
    id: {
      presenceRequired: (0, _ramda.propSatisfies)(_ispresent2.default, "id"),
      stringType: (0, _ramda.propSatisfies)((0, _istype2.default)("String"), "id"),
      bePopulated: (0, _ramda.propSatisfies)((0, _ramda.both)(_ispresent2.default, _ispopulated2.default), "id")
    },
    type: {
      presenceRequired: (0, _ramda.propSatisfies)(_ispresent2.default, "type"),
      stringType: (0, _ramda.propSatisfies)((0, _istype2.default)("String"), "type"),
      bePopulated: (0, _ramda.propSatisfies)((0, _ramda.both)(_ispresent2.default, _ispopulated2.default), "type")
    },
    attributes: {
      presenceRequired: (0, _ramda.propSatisfies)(_ispresent2.default, "attributes"),
      objectType: (0, _ramda.propSatisfies)((0, _istype2.default)("Object"), "attributes"),
      bePopulated: (0, _ramda.propSatisfies)((0, _ramda.both)(_ispresent2.default, _ispopulated2.default), "attributes")
    }
  }
});

(0, _tap.test)(function (_ref) {
  var include = _ref.include,
      end = _ref.end;

  var raw = {
    id: "1",
    type: "accounts",
    attributes: {
      "name": "Kurtis Rainbolt-Greene",
      "age": "27",
      "email": "kurtis@amazon.com",
      "created-at": "2013-02-04T10:35:24-08:00"
    }
  };

  include(accountResource(raw), {
    id: "1",
    type: "accounts",
    attributes: {
      username: "kurtis",
      email: "kurtis@amazon.com",
      name: "Kurtis Rainbolt-Greene",
      age: 27,
      createdAt: new Date("2013-02-04T10:35:24-08:00")
    }
  });
  end();
});